// ===========================================================================
// ==              Player class : implementation                            ==
// ==              Atchoum et X. Pinguin 10 April 2000                      ==
// ===========================================================================
#include <SDL.h>

#include "Actors/Player.h"
#include "Data/ActionId.h"
#include "Data/DataRoom.h"
#include "Render/RenderPlayer.h"
#ifdef _DEBUG
#include <assert.h>
#endif

Player::Player(DataRoom& room, int posx, int posy, int energy) :
  Actor(posx,posy),
  _dx(0), _dy(0),
  _energy(energy)
{
  _renderObj = new RenderPlayer;
  _renderObj->setPosition(posx<<5, posy<<5);
  _data.setFaction(1);
  room.getDynData().push(_data, posx, posy);
  _canBeControled = true;
  _hasMoved = false;
}
// ===========================================================================
Player::Player(int x, int y, int energy) :
  Actor(x,y),
  _dx(0), _dy(0),
  _energy(energy)
{  
  _canBeControled = true;
}
// ===========================================================================
void Player::move()
{
  _nact++;
  switch(_data.getState())
  {
  case MOVELEFT:
    _dx -= 4;
    break;
  case MOVERIGHT:
    _dx += 4;
    break;
  case MOVEUP:
    _dy -= 4;
    break;
  case MOVEDOWN:
    _dy += 4;
    break;
  default:
    break;
  }
  if (_nact&8)
  {
    switch(_data.getState())
    {
    case MOVELEFT:
      _data.move(-1,0);
      break;
    case MOVERIGHT:
      _data.move(1,0);
      break;
    case MOVEUP:
      _data.move(0,-1);
      break;
    case MOVEDOWN:
      _data.move(0,1);
      break;
    default:
      break;
    }
    _data.setState(DONOTHING);
    _canBeControled = true;
    _dx = 0;
    _dy = 0;
    _nact = 0;
    _hasMoved = true;
  }
  else
    _hasMoved = false;
}
// ===========================================================================
void Player::doAction()
{
  if (_data.getState()!=DONOTHING)
  {
    move();
    _renderObj->setPosition((_data.getXPos()<<5)+_dx, (_data.getYPos()<<5)+_dy);
  }
}
// ===========================================================================
bool
Player::mustRedraw() const
{
  return true;
}
// ===========================================================================
bool 
Player::setAction(int action)
{
#ifdef _DEBUG
  // DEBUG
  assert((action>=0)&&(action<=4));
  // END DEBUG
#endif

  bool flag = true;
  flag &= Actor::setAction(action);
  if (action != DONOTHING)
  {
    _canBeControled = false;
    decEnergy();
  }
  else
  {
    _canBeControled = true;
  }

	return flag;
}
// ===========================================================================
bool
Player::setAction()
{
  static int oldAction = DONOTHING;
  if (canControling())
  {
    _hasMoved = false;
    // Pour l'instant, je mets la gestion du clavier ici, mais il va falloir
    // faire une classe permettant de grer (  l'aide d'un Thread, pourquoi pas ? )
    // la gestion du clavier ( vnements  grer ).
    int action = oldAction;
    SDL_Event event;
    while(SDL_PollEvent(&event))
    {
      switch(event.type)
      {
      case SDL_KEYDOWN:
      {
	switch(event.key.keysym.sym)
	{
	case SDLK_UP:
	  action = MOVEUP;
	  break;
	case  SDLK_RIGHT:
	  action = MOVERIGHT;
	  break;
	case  SDLK_DOWN:
	  action = MOVEDOWN;
	  break;
	case  SDLK_LEFT:
	  action = MOVELEFT;
	  break;
        case  SDLK_ESCAPE:
	  _energy = 0;
	  break;
	default:
	  break;
	}
      }
      break;
      case SDL_KEYUP:
	action = DONOTHING;
	break;
      default:
	break;
      }
    }
    oldAction = action;
    return setAction(action);
  }
  return true;
}
// ===========================================================================
Actor*
Player::clone(DataRoom& room, int x, int y, int) const
{
  return new Player(room, x, y, _energy);
}
