// ===========================================================================
// ==                  Class Font : Definition                              ==
// ==               Atchoum & X. Pinguin 6 Avril 2000                       ==
// ===========================================================================
#ifndef _FONT_H_
#define _FONT_H_

// Forward Declaraction
class Sprite;

// Include declaration
#include <assert.h>
#include <string>

/**
   Classe encapsulant la gestion des fontes pour l'affichage des charactres 
   l'cran.
*/
class Font
{
public:
  /** Constructeur.

      @param fontName Le nom du fichier contenant les images de la fonte.
      @param dirName ( optionnel ) : Le rpertoire o se trouve la fonte.
                     Si ce paramtre est omis, on va chercher la fonte dans
		     le repertoire dfini par la variable d'environnement
		     GAMEFONTDIR
  */
  Font( const std::string& fontName );
  Font( const std::string& fontName,
        const std::string& dirName );
  /**
     Constructeur de copie. On ne recopie pas  proprement parl la fonte
     mais on incrmente un compteur de refrence.
   */
  Font(const Font& font);
  /**
     Destructeur : Dcrmente le compteur de rfrence et si ce dernier atteint
     0 on dtruit effectivement la fonte.
  */
  ~Font();

  /**
     Oprateur de copie. Mme principe que le constructeur de copie.
  */
  const Font& operator= (const Font& font);

  // Mthode publique pour les autres classes.
  Sprite& getSpr() const;


  // Largeur de la fonte.
  int getWidth() const;
  // Hauteur de la fonte.
  int getHeight() const;
private:
  // Compteur de rfrence
  int*    _cptRef;
  // Sprite contenant la fonte.
  mutable Sprite* _ptrSpr;
}; // Fin de dclaration de la classe Font.
// ============== INLINE SECTION =============================================
inline Sprite&
Font::getSpr() const
{
  // DEBUG SESSION.
  assert(_ptrSpr != NULL);
  return *_ptrSpr;
}
// ===========================================================================
#endif
