// ========================================================================
// ==              Class Image : Definition                              ==
// ==              Atchoum & X Pinguin 16 Fevrier 2000                   ==
// ========================================================================
#ifndef _IMAGE_H_
#define _IMAGE_H_

#include <SDL.h>
#include <string>

class Screen;

/**
   Classe encapsulant une image Bmp. On y a ajout une possibilit
   de dfinir une couleur transparente.
 */
class Image
{
 public:
  /** Constructeur.
      
      @param filename Le nom du fichier contenant l'image bitmap.
  */
  Image(const std::string& filename);
  /** Constructeur de copie. On partage en fait la surface avec les
      deux images. 
      On gre pour cela un compteur de reference.
  */
  Image(const Image& img);
  /** Destructeur. */
  ~Image();
  /** Operateur d'assignation. Mme remarque que pour le constructeur de copie. */
  const Image& operator=(const Image& img);
  /** Mets la couleur du pixel (fromx,fromy) comme transparent. */
  void setTransparentPixel(int fromx, int fromy);
  /** Mets la couleur indxe pel comme transparent. Cette fonction n'a de sens
      que pour un cran en 256 couleurs. Pour un cran avec plus de couleurs,
      les effets sont imprivisibles ! */
  void setTransparentPixel(Uint8 pel);
  /**
     Particularit SDL. On envoie pour image la surface cre
     par SDL.
   */
  SDL_Surface& getFrame() const;
  /** Lire la couleur du pixel (x,y) */
  Uint32 readPixel(int x, int y) const;
 private:
  // La surface stocke par SDL.
  mutable SDL_Surface* _image;
  // Le compteur de rfrence.
  int* _cptRef;
  friend Screen& operator <<(Screen& scr, const Image& img);
};  

// ================ INLINE SECTION ========================================
inline SDL_Surface&
Image::getFrame() const
{
  return *_image;
}
// ========================================================================

#endif
