// ===========================================================================
// ==                                                                       ==
// ==                                                                       ==
// ===========================================================================
#include <iostream>
#include <stdlib.h>

#include "Graphism/Screen.h"

Screen::Screen(bool fullscreen) :
  _fullscreen(fullscreen),_screen(NULL)
{
  SDL_Init(SDL_INIT_VIDEO); 
  Uint32 flags=SDL_HWSURFACE;
  flags|=fullscreen?SDL_FULLSCREEN:0;
  SDL_PixelFormat * pf=(SDL_GetVideoInfo())->vfmt;
  SDL_Rect ** resDisp=SDL_ListModes(pf,flags);
  _screen=SDL_SetVideoMode((*resDisp)->w,(*resDisp)->h,pf->BitsPerPixel,flags);
  if (!_screen)
    {
	  std::cerr<<"Probleme d'initialisation automatique"<<std::endl;
      exit(EXIT_FAILURE);
    }
    _buffer = SDL_CreateRGBSurface(SDL_SWSURFACE, 
	                               _screen->w,_screen->h,
								   pf->BitsPerPixel,
								   pf->Rmask,
								   pf->Gmask,
								   pf->Bmask,
								   pf->Amask);
#ifdef VERBOSE
  verbose();
#endif
  delete(pf);
  delete(resDisp);
}
// ========================================================================
Screen::Screen(unsigned int width, unsigned int height,
	       unsigned int bpp, bool fullscreen) :
  _fullscreen(fullscreen), _screen(NULL)
{
  SDL_Init(SDL_INIT_VIDEO);
  Uint32 flags=SDL_HWSURFACE;
    flags |=fullscreen?SDL_FULLSCREEN:0;
  int resultOf=SDL_VideoModeOK(width,height,bpp,flags);
  if (resultOf==0)
    {
	  std::cout<<"impossible d'initialiser l'ecran"<<std::endl;
      exit(1);
    }
  else if (resultOf==bpp)
    {
      std::cout<<"OK le mode est accessible"<<std::endl;
    }
  else
    {
      std::cout<<"impossible d'initialiser l'ecran dans ce mode"<<std::endl;
      std::cout<<"initialisation propos avec un bpp de "<<resultOf<<std::endl;
    }
  _screen=SDL_SetVideoMode(width,height,bpp,flags);
  if (!_screen)
    {
      std::cout<<"probleme grave d'initialisation de l'ecran"<<std::endl;
      exit(1);
    }
  SDL_PixelFormat * pf=(SDL_GetVideoInfo())->vfmt;
      _buffer = SDL_CreateRGBSurface(SDL_SWSURFACE, 
	                               _screen->w,_screen->h,
								   pf->BitsPerPixel,
								   pf->Rmask,
								   pf->Gmask,
								   pf->Bmask,
								   pf->Amask);

#ifdef VERBOSE
  verbose();
#endif
}
// ========================================================================
void Screen::verbose(void) const
{
  std::cout<<"*** Information relative  l'environnement graphique ***"<<std::endl;
  std::cout<<"Rsolution Horizontale disponible: "<<_screen->w<<std::endl;
  std::cout<<"Rsolution Verticale disponible: "<<_screen->h<<std::endl;
  std::cout<<"Nombre de bits par pixel: "<<(int)((_screen)->format)->BitsPerPixel<<std::endl;
  std::cout<<"Plein ecran ";
  if (_screen->flags&&SDL_FULLSCREEN)
    {
      std::cout<<"OUI"<<std::endl;
    }
  else
    {
      std::cout<<"NON"<<std::endl;
    }
}
// ========================================================================
unsigned int Screen::readHeight() const
{
  return _screen->h;
}
// ========================================================================
unsigned int Screen::readWidth() const
{
  return _screen->w;
}
// ========================================================================
Screen::~Screen()
{
  if (_screen)
  {
    delete(_screen);
  }  
}
// ========================================================================
void Screen::directWrite(unsigned int X, unsigned int Y, Uint8 red, Uint8 green, Uint8 blue)
{
  if (X>(_screen->w) || Y>(_screen->h))
    {
      return;
    }
  Uint32   pixel;
  Uint8   *bits;
  
  /* Map the color yellow to this display (R=0xFF, G=0xFF, B=0x00)
     Note:  If the display is palettized, you must set the palette first.
  */
  pixel = SDL_MapRGB(_screen->format, (Uint8)red, (Uint8)green,(Uint8)blue);

  /* Calculate the framebuffer offset of the center of the _screen */
  if ( SDL_MUSTLOCK(_screen) ) {
    if ( SDL_LockSurface(_screen) < 0 )
      return;
  }
  
  bits = ((Uint8 *)_screen->pixels)+Y*_screen->pitch+X*_screen->format->BytesPerPixel;
  
  switch(_screen->format->BytesPerPixel) {
  case 1:
    *((Uint8 *)(bits)) = (Uint8)pixel;
    break;
  case 2:
    *((Uint16 *)(bits)) = (Uint16)pixel;
    break;
  case 3: { /* Format/endian independent */
    Uint8 r, g, b;
    
    r = (pixel>>_screen->format->Rshift)&0xFF;
    g = (pixel>>_screen->format->Gshift)&0xFF;
    b = (pixel>>_screen->format->Bshift)&0xFF;
    *((bits)+_screen->format->Rshift/8) = r; 
    *((bits)+_screen->format->Gshift/8) = g;
    *((bits)+_screen->format->Bshift/8) = b;
  }
  break;
  case 4:
    *((Uint32 *)(bits)) = (Uint32)pixel;
    break;
  }
  
  return;
}
// ===========================================================================
void Screen::blit()
{
  if (SDL_MUSTLOCK(_screen))
  {
    SDL_UnlockSurface(_screen);
  }
  SDL_BlitSurface(_buffer, NULL, _screen, NULL);
  SDL_UpdateRect(_screen, 0, 0, 0, 0);
}
// ===========================================================================
void
Screen::clear()
{
  if (SDL_MUSTLOCK(_screen))
    {
      SDL_UnlockSurface(_screen);
    }
  Uint32   pixel;
//  Uint8   *bits;
  
  /* Map the color yellow to this display (R=0xFF, G=0xFF, B=0x00)
     Note:  If the display is palettized, you must set the palette first.
  */
  pixel = SDL_MapRGB(_screen->format, (Uint8)0, (Uint8)0,(Uint8)0);
  SDL_Surface* frameSet = _screen;
  SDL_FillRect(frameSet,NULL,pixel);
}
