// ===========================================================================
// ==                                                                       ==
// ==                                                                       ==
// ===========================================================================
#include <iostream>
#include <assert.h>
#include <SDL.h>

#include "Graphism/Sprite.h"
#include "Graphism/Screen.h"

Sprite::Sprite(Image& img, 
	       int nbrFrame, int nbPosition,
	       int initFrame, 
	       bool isPingPong,
	       int x, int y) :
  _image(img),
  _nbrFrame(nbrFrame), _nbPosition(nbPosition),
  _currentPosition(0), _currentFrame(initFrame),
  _isPingPong(isPingPong)
{
  // DEBUG
  assert(_image.getFrame().w % nbrFrame == 0 );
  // END DEBUG

  SDL_Surface& frameSet = _image.getFrame();
 _destRec.x=x;
 _destRec.y=y;
 _destRec.w=(frameSet.w)/nbrFrame;
 _destRec.h=(frameSet.h)/nbPosition;

 _srcRec.w=_destRec.w;
 _srcRec.h=_destRec.h;
 _srcRec.x=initFrame*_srcRec.w;
 _srcRec.y=0*_srcRec.h;
 _incFrame = 1;
}
// ===========================================================================
void Sprite::nextFrame()
{
  _currentFrame += _incFrame;
  if (_currentFrame == _nbrFrame)
  {
    if (!_isPingPong)
      _currentFrame = 0;
    else
    {
      _currentFrame = _nbrFrame-2;
      _incFrame = -1;
    }
  }
  if (_currentFrame == -1)
  {
    _currentFrame = 1;
    _incFrame = 1;
  }
  _srcRec.x = _currentFrame*_srcRec.w;
  /*  if (_currentFrame!=0)
  {
    _srcRec.x+=_srcRec.w;
  }
  else
  {
    _srcRec.x=0;
    }*/
}
// ===========================================================================
void Sprite::setNewPos(int x, int y)
{
  _destRec.x=x;
  _destRec.y=y;
}
// ===========================================================================
void Sprite::setFrame(int initFrame)
{
  assert(initFrame>=0);
  assert(initFrame<_nbrFrame);
  _currentFrame = initFrame;
  _srcRec.x = _currentFrame*_srcRec.w;
}
// ===========================================================================
void Sprite::setPosition(int position)
{
  assert(position>=0);
  assert(position<_nbPosition);
  _currentPosition = position;
  _srcRec.y = _currentPosition*_srcRec.h;
}
// ===========================================================================
Screen& 
operator<<(Screen& scr, const Sprite& spr)
{
  SDL_BlitSurface(&(spr.getImage().getFrame()), 
		  spr.getSrc(), scr.getScreen(), 
		  spr.getDst());
  return scr;
}
