// =============================================================================
// ==                    class ModPlayer : Implementation.                    ==
// ==                   Atchoum & X. Pinguin    3 Mars 2000                   ==
// =============================================================================
#include "Music/ModPlayer.h"

#include <unistd.h>
#include <mikmod.h>
#include <SDL_thread.h>
#include <iostream>

ModPlayer::ModPlayer() :  _module(NULL), _isQuitting(false),
			  _thread(NULL)
{
  /* register all the drivers */
  MikMod_RegisterAllDrivers();

  /* register all the module loaders */
  MikMod_RegisterAllLoaders();
  
  /* initialize the library */
  // Ncessaire avec Linux ????
  //  md_mode |= DMODE_SOFT_MUSIC;
  if (MikMod_Init("")) 
  {
    cerr << "Could not initialize sound, reason: "
	 << MikMod_strerror(MikMod_errno) << endl;
    exit(-1);
  }
}
// ========================================================================
ModPlayer::~ModPlayer()
{

  /* give up */
  MikMod_Exit();

  _thread = NULL;
}
// ========================================================================
bool
ModPlayer::load(const string& name)
{
  bool flag = true;
  if (isPlaying())
  {
    stop();
  }
  if (_module)
    Player_Free(_module);

  /* load module */
  _module = Player_Load((char*)name.c_str(), 64, 0);

  if (!_module)
  {
    cerr << "Could not load module, reason: "
	 << MikMod_strerror(MikMod_errno) << endl;
    flag = false;
  }
  return flag;
}
// ========================================================================
bool
ModPlayer::play()
{
  _isQuitting = false;
  if (isPlaying())
    stop();
  if (_module)
    _thread = SDL_CreateThread(PlayModule, this);
  return (_thread!=NULL);
}
// ========================================================================
bool
ModPlayer::stop()
{
  bool flag = pause();
  rewind();
  return (flag);
}
// ========================================================================
bool
ModPlayer::pause()
{
  int status;
  if (_thread != NULL)
  {
    _isQuitting = true;
    SDL_WaitThread( _thread, &status);

    _thread = NULL;
  }
  return (status==1);
}
// ========================================================================
void
ModPlayer::rewind()
{
  bool isplay = isPlaying();
  if (isplay)
  {
    pause();
  }
  if (_module)
  {
    Player_PrevPosition();
  }
  if (isplay)
  {
    play();
  }
}
// ========================================================================
bool
ModPlayer::isPlaying()
{
  return (!_isQuitting)&&(Player_Active());
}
// ========================================================================
int
ModPlayer::PlayModule(void* ptMod)
{
  ModPlayer* mod = (ModPlayer*)ptMod;
  Player_Start(mod->_module);
  while((mod->isPlaying())&&(!mod->_isQuitting))
  {
    usleep(10000);
    MikMod_Update();
  }
  Player_Stop();

  return 1;
}
// ========================================================================
