// ===========================================================================
// ==                                                                       ==
// ==                                                                       ==
// ===========================================================================

#include "Render/RenderRoom.h"
#include "Render/RenderActor.h"
#include "Render/RenderObject.h"
#include "Render/RenderEmpty.h"
#include "Graphism/Screen.h"

RenderRoom::RenderRoom(int lx, int ly, int lz) :
  _staticRendering(lx,ly,lz),
  _dynamicRendering(lx,ly,lz)
{
}
// ===========================================================================
RenderRoom::~RenderRoom()
{
}
// ===========================================================================
Screen&
RenderRoom::render(Screen& scr)
{
	int i,j;

  // 1st Step : Search redrawable dynamic Object
  //            and set neighbourgh static Object
  //            to redraw.
  // ============================================
  for ( i = 0; i < _dynamicRendering.getXLength(); i++)
    for ( j = 0; j < _dynamicRendering.getYLength(); j++)
    {
      RenderActor* rendObj = _dynamicRendering.getElt(i,j);
      if ((rendObj!=NULL) &&
	  (rendObj->mustRedraw()) )
      {
	RenderObject* rendSta = _staticRendering.getElt(i,j);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	//RenderObject* 
	rendSta = _staticRendering.getElt(i-1,j);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _staticRendering.getElt(i+1,j);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _staticRendering.getElt(i,j-1);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _staticRendering.getElt(i,j+1);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
      }
    }
   // 2.1nd Step : Redraw neighbour actors.
	// ===================================
  for (i = 0; i < _dynamicRendering.getXLength(); i++)
    for ( j = 0; j < _dynamicRendering.getYLength(); j++)
    {
      RenderActor* rendObj = _dynamicRendering.getElt(i,j);
      if ((rendObj!=NULL) &&
	  (rendObj->mustRedraw()) )
      {
		RenderActor* rendSta = _dynamicRendering.getElt(i,j);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _dynamicRendering.getElt(i-1,j);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _dynamicRendering.getElt(i+1,j);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _dynamicRendering.getElt(i,j-1);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
	rendSta = _dynamicRendering.getElt(i,j+1);
	if (rendSta != NULL)
	  rendSta->setMustRedraw();
      }
    }

  // 2nd Step : Redraw Static Object.
  // ===========================================
  for (i = 0; i < _staticRendering.getXLength(); i++)
    for ( j = 0; j < _staticRendering.getYLength(); j++)
    {
      RenderObject* rendObj = _staticRendering.getElt(i,j);
      if ((rendObj!=NULL) &&
	  (rendObj->mustRedraw()) )
	scr << (*rendObj);
    }

  // 3rd Step : Redraw dynamic Object.
  // ===========================================
  for (i = 0; i < _dynamicRendering.getXLength(); i++)
    for ( j = 0; j < _dynamicRendering.getYLength(); j++)
    {
      RenderBase* rendActor    = _dynamicRendering.getElt(i,j);
      if ((rendActor!=NULL) &&
	  (rendActor->mustRedraw()) )
	scr << (*rendActor);
    }
  return scr;
}
// ===========================================================================
void
RenderRoom::addObject(RenderObject& obj, int x, int y, int z)
{
  _staticRendering.setElt(&obj,x,y,z);
}
// ===========================================================================
bool
RenderRoom::removeObject(int x, int y, int z)
{
  RenderObject* obj = _staticRendering.getElt(x,y,z);
  _staticRendering.setElt(NULL,x,y,z);
  return (obj!=NULL);
}
// ===========================================================================
void
RenderRoom::addActor(RenderActor& actor, int x, int y, int z)
{
  _dynamicRendering.setElt(&actor, x, y, z);
  actor.setPosition((x<<5),(y<<5));
}
// ===========================================================================
bool
RenderRoom::removeActor(int x, int y, int z)
{
  RenderActor* actor = _dynamicRendering.getElt(x,y,z);
  _dynamicRendering.setElt(NULL,x,y,z);
  return (actor != NULL);
}
